module save_snapshot_m

  implicit none

contains

  subroutine save_snapshot(s, corner, step, nlon, nlat, copy, d)

    ! Libraries:
    use jumble, only: arth

    use derived_types, only: snapshot, shpc_slice_handler
    use shpc_close_m, only: shpc_close
    use shpc_create_m, only: shpc_create
    use write_extr_map_m, only: write_extr_map
    use write_snapshot_m, only: write_snapshot

    type(snapshot), intent(in):: s

    real, intent(in):: corner(:) ! (2)
    ! longitude and latitude of the corner of the whole grid in input
    ! NetCDF, in degrees

    real, intent(in):: step(:) ! (2) ! longitude and latitude steps, in degrees

    integer, intent(in):: nlon, nlat
    ! size of ssh array in input NetCDF, assuming no repeated point if
    ! the grid is global

    integer, intent(in):: copy, d

    ! Local:
    TYPE(shpc_slice_handler) hshpc_cyclo, hshpc_anti
    integer i

    !-----------------------------------------------------------------

    call shpc_create(hshpc_cyclo, shpc_dir = "SHPC", cyclone = .true., &
         slice = 0)
    call shpc_create(hshpc_anti, shpc_dir = "SHPC", cyclone = .false., &
         slice = 0)
    call write_snapshot(s, hshpc_cyclo, hshpc_anti, d)
    CALL shpc_close(hshpc_cyclo)
    CALL shpc_close(hshpc_anti)
    print *, 'Created a slice in SHPC.'
    print *, "Number of extrema:", s%number_extr
    print *, "s%ind_extr:"

    do i = 1, s%number_extr
       print *, s%ind_extr(:, i)
    end do

    call write_extr_map(s%extr_map, longitude = arth(corner(1) - copy &
         * step(1), step(1), nlon + 2 * copy), latitude = arth(corner(2), &
         step(2), nlat))

  end subroutine save_snapshot

end module save_snapshot_m
