program test_read_snapshot

  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use jumble, only: get_command_arg_dyn, new_unit, deg_to_rad, assert, &
       read_opcol
  use shapelib_03, only: dbf_read_attribute_03

  use derived_types, only: snapshot, shpc_slice_handler, shpc_slice_meta
  use read_snapshot_m, only: read_snapshot
  use shpc_close_m, only: shpc_close
  use shpc_open_m, only: shpc_open
  use save_snapshot_m, only: save_snapshot

  implicit none

  character(len = :), allocatable:: shpc_dir
  type(snapshot) s
  TYPE(shpc_slice_handler), allocatable:: hshp(:) ! (n_slices)
  type(shpc_slice_meta), allocatable:: ssm(:) ! (n_slices)
  integer copy, unit, k, i, iostat
  integer n_slices ! number of input SHPC directories

  real:: corner_deg(2) = [0.125, - 59.875]
  ! longitude and latitude of the corner of the whole grid in input
  ! NetCDF, in degrees

  real:: step_deg(2) = [0.25, 0.25] ! longitude and latitude steps, in degrees
  logical periodic ! grid is periodic in longitude

  integer:: nlon = 120, nlat = 120
  ! size of ssh array in input NetCDF, assuming no repeated point if
  ! the grid is global

  integer:: dist_lim = 12
  ! We look for an overlapping eddy at dist_lim (in grid points) of
  ! the first extremum.

  namelist /main_nml/ dist_lim, k
  namelist /grid_nml/ corner_deg, step_deg, nlon, nlat

  !-------------------------------------------------------------------------

  call get_command_arg_dyn(1, shpc_dir, "Required argument: SHPC-directory")

  call new_unit(unit)
  open(unit, file = shpc_dir // "/grid_nml.txt", status = "old", &
       action = "read", position  = "rewind")
  read(unit, nml = grid_nml)
  close(unit)

  ! n_slices:

  open(unit, file = shpc_dir // "/n_slices.txt", status = "old", &
       action = "read", position = "rewind", iostat = iostat)

  if (iostat == 0) then
     read(unit, fmt = *) n_slices
     close(unit)
  else
     print *, "test_read_snapshot: Could not open ", shpc_dir, "/n_slices.txt"
     print *, "test_read_snapshot: Assuming there is only one slice..."
     n_slices = 1
  end if

  allocate(hshp(n_slices), ssm(n_slices))

  do i = 1, n_slices
     call shpc_open(hshp(i), trim(shpc_dir), cyclone = .false., slice = i - 1, &
          pszaccess = "rb")
     call dbf_read_attribute_03(ssm(i)%d0, hshp(i)%extremum, &
          hshp(i)%extr_date, ishape = 0)
     call read_opcol(ssm(i)%ishape_last, hshp(i)%unit, my_lbound = ssm(i)%d0)
  end do

  ! main_nml:
  k = ssm(1)%d0 ! default value
  write(unit = error_unit, nml = main_nml)
  write(unit = error_unit, fmt = *) "Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)

  ! As we are requiring the grid spacing to be uniform, the value of
  ! "periodic" may be deduced from the values of step_deg(1) and nlon:
  periodic = nint(360. / step_deg(1)) == nlon
  print *, "test_read_snapshot: periodic = ", periodic
  if (periodic) call assert(2 * dist_lim * step_deg(1) < 180., &
       "test_read_snapshot dist_lim")
  copy = merge(dist_lim, 0, periodic)

  call read_snapshot(s, hshp, ssm, nlon, nlat, k, &
       corner = corner_deg * deg_to_rad, step = step_deg * deg_to_rad, &
       copy = copy)

  do i = 1, n_slices
     CALL shpc_close(hshp(i))
  end do

  call save_snapshot(s, corner_deg, step_deg, nlon, nlat, copy, k)

end program test_read_snapshot
